# Gas analysis

## `elliptic-curve-solidity` library

Gas consumption of all library functions:

```bash
·---------------------------------------|---------------------------|-------------|----------------------------·
|  Solc version: 0.5.8+commit.23d335f2  ·  Optimizer enabled: true  ·  Runs: 200  ·  Block limit: 6721975 gas  │
········································|···························|·············|·····························
|  Methods                              ·               20 gwei/gas               ·       335.29 usd/eth       │
·····················|··················|·············|·············|·············|··············|··············
|  Contract          ·  Method          ·  Min        ·  Max        ·  Avg        ·  # calls     ·  usd (avg)  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  deriveY         ·      50504  ·      59058  ·      54858  ·           6  ·       0.37  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  ecAdd           ·      29408  ·      61603  ·      48298  ·          24  ·       0.32  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  ecInv           ·      27196  ·      28092  ·      27644  ·           4  ·       0.19  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  ecMul           ·      52150  ·     674698  ·     137948  ·          29  ·       0.93  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  ecSub           ·      55576  ·      61782  ·      58907  ·           8  ·       0.40  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  invMod          ·      23242  ·      52736  ·      37591  ·           8  ·       0.25  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  isOnCurve       ·      28727  ·      30973  ·      29866  ·           8  ·       0.20  │
·····················|··················|·············|·············|·············|··············|··············
|  EllipticCurve     ·  toAffine        ·      56817  ·      56892  ·      56855  ·           4  ·       0.38  │
·····················|··················|·············|·············|·············|··············|··············
|  Migrations        ·  setCompleted    ·          -  ·          -  ·      26939  ·           1  ·       0.18  │
·····················|··················|·············|·············|·············|··············|··············
|  Deployments                          ·                                         ·  % of limit  ·             │
········································|·············|·············|·············|··············|··············
|  EllipticCurve                        ·          -  ·          -  ·     822657  ·      12.2 %  ·       5.52  │
········································|·············|·············|·············|··············|··············
|  Migrations                           ·          -  ·          -  ·     188570  ·       2.8 %  ·       1.26  │
·---------------------------------------|-------------|-------------|-------------|--------------|-------------·
```

Gas consumption of `Secp256k1` derive key operation:

```bash
·---------------------------------------|---------------------------|-------------|----------------------------·
|  Solc version: 0.5.8+commit.23d335f2  ·  Optimizer enabled: true  ·  Runs: 200  ·  Block limit: 6721975 gas  │
········································|···························|·············|·····························
|  Methods                              ·               20 gwei/gas               ·       335.32 usd/eth       │
····················|···················|·············|·············|·············|··············|··············
|  Contract         ·  Method           ·  Min        ·  Max        ·  Avg        ·  # calls     ·  usd (avg)  │
····················|···················|·············|·············|·············|··············|··············
|  Secp256k1        ·  deriveKey        ·     528127  ·     572545  ·     552976  ·          18  ·       3.71  │
····················|···················|·············|·············|·············|··············|··············
|  Deployments                          ·                                         ·  % of limit  ·             │
········································|·············|·············|·············|··············|··············
|  EllipticCurve                        ·          -  ·          -  ·     822657  ·      12.2 %  ·       5.52  │
········································|·············|·············|·············|··············|··············
|  Example                              ·          -  ·          -  ·     859078  ·      12.8 %  ·       5.76  │
········································|·············|·············|·············|··············|··············
|  Migrations                           ·          -  ·          -  ·     188570  ·       2.8 %  ·       1.26  │
·---------------------------------------|-------------|-------------|-------------|--------------|-------------·
```

## Constructor-based library

Note: this analysis has been performed for comparison purposes with the current library implementation.

Gas consumption of all library functions:

```bash
·---------------------------------------|---------------------------|-------------|----------------------------·
|  Solc version: 0.5.8+commit.23d335f2  ·  Optimizer enabled: true  ·  Runs: 200  ·  Block limit: 6721975 gas  │
········································|···························|·············|·····························
|  Methods                              ·               20 gwei/gas               ·       334.90 usd/eth       │
······················|·················|·············|·············|·············|··············|··············
|  Contract           ·  Method         ·  Min        ·  Max        ·  Avg        ·  # calls     ·  usd (avg)  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  deriveKey      ·    2449631  ·    2644010  ·    2561220  ·          18  ·      17.15  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  deriveY        ·      48232  ·      59098  ·      53229  ·           6  ·       0.36  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  ecAdd          ·      27993  ·     157627  ·     103632  ·          24  ·       0.69  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  ecInv          ·      26322  ·      26386  ·      26354  ·           4  ·       0.18  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  ecMul          ·     129364  ·    3047648  ·     497540  ·          26  ·       3.33  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  ecSub          ·     132938  ·     158168  ·     145647  ·           8  ·       0.98  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  invMod         ·      23280  ·     141814  ·      80799  ·           8  ·       0.54  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  isOnCurve      ·      27939  ·      28673  ·      28322  ·           8  ·       0.19  │
······················|·················|·············|·············|·············|··············|··············
|  EllipticCurve      ·  toAffine       ·     144922  ·     148807  ·     146865  ·           4  ·       0.98  │
······················|·················|·············|·············|·············|··············|··············
|  Deployments                          ·                                         ·  % of limit  ·             │
········································|·············|·············|·············|··············|··············
|  EllipticCurve                        ·     955084  ·     972196  ·     967710  ·      14.4 %  ·       6.48  │
········································|·············|·············|·············|··············|··············
|  Migrations                           ·          -  ·          -  ·     188570  ·       2.8 %  ·       1.26  │
·---------------------------------------|-------------|-------------|-------------|--------------|-------------·
```
